{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
launchplink. Copyright 2004 by Mario A. Valdez-Ramirez.
Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:
The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
unit mainexample;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

CONST
  opmC_Def_SSHExe = 'plink.exe';
  opmC_Def_SSHUsername = 'sshtunnel';
  opmC_Def_SSHPassword = 'tunnel';
  opmC_Def_SSHLocalHost = '127.0.0.1';
  opmC_Def_SSHRemoteHost = 'wintermute';
  opmC_Def_SSHLocalPort = 3306;
  opmC_Def_SSHCompress = 1;
  opmC_Def_SSHProtocol = 2;
  opmC_SSHConnectLapse = 5000; 


type
  TForm1 = class(TForm)
    opm_Button_Start: TButton;
    opm_Button_Stop: TButton;
    opm_LabeledEdit_Username: TLabeledEdit;
    opm_LabeledEdit_Password: TLabeledEdit;
    opm_LabeledEdit_Hostname: TLabeledEdit;
    opm_Button_StartHidden: TButton;
    opm_Button_CheckStall: TButton;
    opm_Button_StopAny: TButton;
    procedure FormShow(Sender: TObject);
    procedure opm_Button_StartClick(Sender: TObject);
    procedure opm_Button_StartHiddenClick(Sender: TObject);
    procedure opm_Button_StopClick(Sender: TObject);
    procedure opm_Button_CheckStallClick(Sender: TObject);
    procedure opm_Button_StopAnyClick(Sender: TObject);
  private
  public
  end;

VAR
  SSHTunnelHandle : CARDINAL;
  Form1: TForm1;


FUNCTION FNopm_OpenSSHTunnel (SSHLHost, SSHRHost : STRING; SSHLPort, SSHRPort : WORD; SSHUser, SSHPass : STRING; VisibleWindow : INTEGER) : BOOLEAN;
FUNCTION FNopm_CheckStallTunnel : BOOLEAN;
FUNCTION FNopm_RunExternalApp (ExeFileName, ExeParams, RunDirectory : STRING; ShowApp : BOOLEAN; WaitForIdle : LONGINT): CARDINAL;
PROCEDURE PRopm_StopExternalApp (AppHandle : CARDINAL);
PROCEDURE PRopm_StopExternalAppByName (AppTitle : STRING);
FUNCTION FNopm_IsAppRunning (AppTitle : STRING) : BOOLEAN;


implementation

{$R *.dfm}

USES ShellApi;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNopm_OpenSSHTunnel (SSHLHost, SSHRHost : STRING; SSHLPort, SSHRPort : WORD; SSHUser, SSHPass : STRING; VisibleWindow : INTEGER) : BOOLEAN;
VAR
  PlinkParams : STRING;
BEGIN
  PLinkParams := '-ssh -' + INTTOSTR (opmC_Def_SSHProtocol);
  IF (opmC_Def_SSHCompress > 0) THEN PLinkParams := PLinkParams + ' -C';
  PLinkParams := PLinkParams + ' -l ' + SSHUser + ' -pw ' + SSHPass;
  PLinkParams := PLinkParams + ' -L ' + INTTOSTR (SSHLPort) + ':' + SSHLHost + ':' + INTTOSTR (SSHRPort);
  PLinkParams := PLinkParams + ' ' + SSHRHost + '';
  IF (VisibleWindow > 0) THEN
    BEGIN
      MessageDlg ('About to execute: ' + ExtractFilePath (Application.Exename) + opmC_Def_SSHExe + ' ' + PLinkParams, mtInformation, [mbOk], 0);
      SSHTunnelHandle := FNopm_RunExternalApp (ExtractFilePath (Application.Exename) + opmC_Def_SSHExe,
                         PLinkParams, ExtractFilePath (Application.Exename), TRUE, opmC_SSHConnectLapse);
    END
  ELSE
    SSHTunnelHandle := FNopm_RunExternalApp (ExtractFilePath (Application.Exename) + opmC_Def_SSHExe,
                       PLinkParams, ExtractFilePath (Application.Exename), FALSE, opmC_SSHConnectLapse);
  FNopm_OpenSSHTunnel := (SSHTunnelHandle > 0);
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNopm_CheckStallTunnel : BOOLEAN;
BEGIN
  FNopm_CheckStallTunnel := (FNopm_IsAppRunning (opmC_Def_SSHExe) = TRUE);
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNopm_RunExternalApp (ExeFileName, ExeParams, RunDirectory : STRING; ShowApp : BOOLEAN; WaitForIdle : LONGINT): CARDINAL;
VAR
  ExeInfo : TShellExecuteInfo;
BEGIN
  ExeInfo.cbSize := SIZEOF (ExeInfo);
  ExeInfo.fMask := (SEE_MASK_NOCLOSEPROCESS OR SEE_MASK_FLAG_NO_UI);
  ExeInfo.wnd := Application.Handle;
  ExeInfo.lpVerb := 'open';
  ExeInfo.lpFile := PCHAR (ExeFileName);
  ExeInfo.lpParameters := PCHAR (ExeParams);
  ExeInfo.lpDirectory := PCHAR (RunDirectory);
  IF (ShowApp = FALSE) THEN
    ExeInfo.nShow := SW_HIDE    // SW_SHOWMINNOACTIV ???
  ELSE
    ExeInfo.nShow := SW_SHOWNORMAL;    // SW_SHOWDEFAULT ???
  IF (ShellExecuteEx (@ExeInfo) = TRUE) THEN
    BEGIN
      IF (WaitForIdle > 0) THEN WaitForInputIdle (ExeInfo.hProcess, WaitForIdle);
      FNopm_RunExternalApp := ExeInfo.hProcess;
    END
  ELSE FNopm_RunExternalApp := 0;
end;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE PRopm_StopExternalApp (AppHandle : CARDINAL);
BEGIN
  IF (AppHandle > 0) THEN
    IF (TerminateProcess (AppHandle, ExitCode) = TRUE)
      THEN CloseHandle (AppHandle);
end;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE PRopm_StopExternalAppByName (AppTitle : STRING);
VAR
  WindowHandle: HWND;
  WindowTitle: ARRAY [0..255] of CHAR;
  ProcID : CARDINAL;
  ProcHandle : CARDINAL;
BEGIN
    WindowHandle := GetWindow (Application.Handle, GW_HWNDFIRST);
    WHILE (WindowHandle > 0) DO
      BEGIN
        FillChar (WindowTitle, LENGTH (WindowTitle), #0);
        GetWindowText (WindowHandle, WindowTitle, LENGTH (WindowTitle) - 1);
        IF (POS (UPPERCASE (AppTitle), UPPERCASE (STRING (WindowTitle))) > 0) THEN
          BEGIN
            GetWindowThreadProcessId (WindowHandle, @ProcID);
            ProcHandle := OpenProcess (PROCESS_TERMINATE, FALSE, ProcID);
            TerminateProcess (ProcHandle, 0);
            CloseHandle (ProcHandle);
            BREAK;
          END
        ELSE WindowHandle := GetWindow (WindowHandle, GW_HWNDNEXT);
      END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNopm_IsAppRunning (AppTitle : STRING) : BOOLEAN;
VAR
  WindowHandle: HWND;
  WindowTitle: ARRAY [0..255] of CHAR;
BEGIN
    FNopm_IsAppRunning := FALSE;
    WindowHandle := GetWindow (Application.Handle, GW_HWNDFIRST);
    WHILE (WindowHandle > 0) DO
      BEGIN
        FillChar (WindowTitle, LENGTH (WindowTitle), #0);
        GetWindowText (WindowHandle, WindowTitle, LENGTH (WindowTitle) - 1);
        IF (POS (UPPERCASE (AppTitle), UPPERCASE (STRING (WindowTitle))) > 0) THEN
          BEGIN
            FNopm_IsAppRunning := TRUE;
            BREAK;
          END
        ELSE WindowHandle := GetWindow (WindowHandle, GW_HWNDNEXT);
      END;
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure TForm1.FormShow(Sender: TObject);
begin
  opm_LabeledEdit_Hostname.Text := opmC_Def_SSHRemoteHost;
  opm_LabeledEdit_Username.Text := opmC_Def_SSHUsername;
  opm_LabeledEdit_Password.Text := opmC_Def_SSHPassword;
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure TForm1.opm_Button_StartClick(Sender: TObject);
begin
  IF (MessageDlg ('This test will close any SSH tunnel currently active. Are you sure you want to proceed with the test?', mtWarning, [mbYes, mbNo], 0) = mrYes) THEN
    BEGIN
      PRopm_StopExternalAppByName (opmC_Def_SSHExe);
      IF (FNopm_OpenSSHTunnel (opmC_Def_SSHLocalHost,
                               TRIM (opm_LabeledEdit_Hostname.Text),
                               opmC_Def_SSHLocalPort, opmC_Def_SSHLocalPort,
                               TRIM (opm_LabeledEdit_Username.Text),
                               TRIM (opm_LabeledEdit_Password.Text), 1) = FALSE) THEN
        BEGIN
          MessageDlg  ('Cannot launch SSH tunnel auxiliar application!', mtWarning, [mbOk], 0);
        END;
    END;
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure TForm1.opm_Button_StartHiddenClick(Sender: TObject);
begin
  IF (MessageDlg ('This test will close any SSH tunnel currently active. Are you sure you want to proceed with the test?', mtWarning, [mbYes, mbNo], 0) = mrYes) THEN
    BEGIN
      PRopm_StopExternalAppByName (opmC_Def_SSHExe);
      IF (FNopm_OpenSSHTunnel (opmC_Def_SSHLocalHost,
                               TRIM (opm_LabeledEdit_Hostname.Text),
                               opmC_Def_SSHLocalPort, opmC_Def_SSHLocalPort,
                               TRIM (opm_LabeledEdit_Username.Text),
                               TRIM (opm_LabeledEdit_Password.Text), 0) = FALSE) THEN
        BEGIN
          MessageDlg  ('Cannot launch SSH tunnel auxiliar application!', mtWarning, [mbOk], 0);
        END
      ELSE MessageDlg  ('SSH tunnel started. PLink HandleID=' + INTTOSTR (SSHTunnelHandle), mtInformation, [mbOk], 0);
    END;
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure TForm1.opm_Button_StopClick(Sender: TObject);
begin
  PRopm_StopExternalApp (SSHTunnelHandle);
  SSHTunnelHandle := 0;
end;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure TForm1.opm_Button_StopAnyClick(Sender: TObject);
begin
  PRopm_StopExternalAppByName (opmC_Def_SSHExe);
  SSHTunnelHandle := 0;
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure TForm1.opm_Button_CheckStallClick(Sender: TObject);
begin
  IF (FNopm_IsAppRunning (opmC_Def_SSHExe) = TRUE) THEN
    MessageDlg ('PLink is running!', mtInformation, [mbOk], 0)
  ELSE
    MessageDlg ('PLink is not running.', mtInformation, [mbOk], 0);
end;


end.
